var http = require('http').createServer(handler);
var fs = require('fs');
var io = require('socket.io')(http);
var sensor = require('./mcp3008.js');
var Gpio = require('onoff').Gpio;

io.set('close timeout', 10);
io.set('heartbeat timeout', 10);

const EventEmitter = require('events');
const rotatorEmitter = new EventEmitter();
var mcp3008 = new sensor.MCP3008();

const version=0.2;
const loopinterval = 100; // ms
var adc1=0;
var DirArr = [0,0,0,0,0]; // 5 * 100ms = 500ms lag
var LastDirection;
var turntimer;
var postitionreports;
var autoturntimer;
const allowedhtml = [ '/', '/socket.io.js', 'rotator.html' ];
// Note. mcp3008 has reserved gpios 5, 9, 10, 11
var relay_cw = new Gpio(17, 'out');
var relay_ccw = new Gpio(27, 'out'); 


http.listen(80);
function handler(req, res) {
  console.log(req.url);
  if (allowedhtml.indexOf(req.url) > -1) {
    if (req.url == '/') filename='/rotator.html'; else filename=req.url;
    fs.readFile(__dirname + '/public' + filename, function(err, data) { 
      if (err) {
        res.writeHead(404, {'Content-Type': 'text/html'});
        return res.end("404 Not Found ");
      }
      var jsregex=/\.js$/i;
      if (filename.match(jsregex)) {
        res.writeHead(200, {'Content-Type': 'application/javascript'});
      }  else {
        res.writeHead(200, {'Content-Type': 'text/html'});
      }
      res.write(data);
      return res.end();
    });
  } else {
    return res.end("404 Not Found ");
  }
}

io.sockets.on('connection', function (socket) {// Web Socket Connection
  let handshake = socket.handshake;
  console.log(handshake.time + " New connection from "+ handshake.address + ". id=" + socket.id);
  
  rotatorEmitter.on('event', function(key,value) {
    //if (key != 'direction') console.log("socket emit key="+key + " value="+ value);
    socket.emit('rotatorsocket',{ key: key, value: value });

  });
  socket.on('rotatorsocket', function(data) { 
    // Someone did sometring on web gui
    if (data.turn !== undefined) {
      switch(data.turn) {
        case "cw":
          clearTimeout(turntimer);
          turntimer = setTimeout(function() { turn("stop") }, 2000);
          turn("cw");
          break;
        case "ccw":
          clearTimeout(turntimer);
          turntimer = setTimeout(function() { turn("stop") }, 2000);
          turn("ccw");
          break;
        case "stop":
          clearTimeout(turntimer);
          clearTimeout(autoturntimer);
          turn("stop");
          break;
        default:
          console.log("Unknown turn command: "+data.turn);
          break;
      } 
    } 
    if (data.goto !== undefined) {
        rotatorEmitter.emit('event', 'goto', parseFloat(data.goto));
        clearTimeout(autoturntimer); // avoid duplicate goto's
        clearTimeout(turntimer);
        turn("stop");
        turn_to_direction(data.goto);
    }
    if (data.getstatus !== undefined) {
      rotatorEmitter.emit('event', 'direction', LastDirection);
    }
    console.log("Received message:");
    console.log(data); 
  });
  socket.on('disconnect',function() {
    console.log('Client disconnected '+socket.id);
  });
});

process.on('SIGINT', function () { //on ctrl+c
  process.exit();
}); 

function adval_to_degrees(val) {
	// AD value to degress
        
        // Curve fitting values y = a+bx+cx^2
        var a=0.001;
        var b=0.3;
        var c=0.009;
        var degs = a+b*val+(c*val*c*val);
	return (degs-17);
}

function turn(direction) {
  switch(direction) {
    case "cw":
      rotatorEmitter.emit('event', 'turn', 'cw');
      relay_cw.writeSync(0);
      break;
    case "ccw":
      rotatorEmitter.emit('event', 'turn', 'ccw');
      relay_ccw.writeSync(0);
      break;
    case "stop":
      rotatorEmitter.emit('event', 'turn', 'stop');
      relay_cw.writeSync(1);
      relay_ccw.writeSync(1);
      break;
    }    
}

function turn_to_direction(minne) {
        var wanted_direction=Math.round(minne);
	var marginaali=5;
	if (LastDirection <= (wanted_direction-marginaali)) {
        	console.log("Selected direction "+wanted_direction+". Direction now "+LastDirection+". Turn clockwise.");
		turn("cw");
		autoturntimer = setTimeout(function() { turn_to_direction(wanted_direction); }, 1000);
	} else if (LastDirection >= (wanted_direction+marginaali)) {
        	console.log("Selected direction "+wanted_direction+". Direction now "+LastDirection+". Turn counter clockwise.");
		turn("ccw");
		autoturntimer = setTimeout(function() { turn_to_direction(wanted_direction); }, 1000);
	} else {
        	console.log("Selected direction "+wanted_direction+". Direction now "+LastDirection+". Inside marginal "+marginaali+". Stop.");
		turn("stop");
		clearTimeout(autoturntimer);
	}
}

function mainloop() {
  var adc1 = mcp3008.readAdc(mcp3008.channels.CHANNEL_1); // ATTENTION
  //var adc1 = Math.floor(Math.random()*360);
  var adcf = parseFloat(adc1);
  DirArr.shift();
  DirArr.push(adcf);
  var DirSum, DirAvg = 0;
  if (DirArr.length) {
    DirSum = DirArr.reduce(function(a, b) { return a + b; });
    DirAvg = DirSum / DirArr.length;
  }
  var CurrDirection = adval_to_degrees(DirAvg).toFixed(0);
  //console.log("adc="+adc1+" avg="+DirAvg+" CurrDirection="+CurrDirection);
  if (CurrDirection != LastDirection) {
      rotatorEmitter.emit('event', 'direction', CurrDirection);
      LastDirection=CurrDirection;
  }
}

relay_cw.writeSync(1);
relay_ccw.writeSync(1);
timer1 = setInterval(mainloop, loopinterval);
console.log("Rotator control version "+version+" started.");
